@@ -410,7 +410,18 @@ class LibspecManager(object):
 
         self._main_thread = threading.current_thread()
 
-        self._observer = watchdog_wrapper.create_observer()
+        watch_impl = os.environ.get("ROBOTFRAMEWORK_LS_WATCH_IMPL", "watchdog")
+        if watch_impl not in ("watchdog", "fsnotify"):
+            log.info(
+                f"ROBOTFRAMEWORK_LS_WATCH_IMPL should be 'watchdog' or 'fsnotify'. Found: {watch_impl} (falling back to fsnotify)"
+            )
+            # i.e.: the default is watchdog, so, if a different one is set,
+            # presumably the default is not ok, so, fall back to watchdog.
+            watch_impl = "fsnotify"
+
+        self._observer = watchdog_wrapper.create_observer(
+            watch_impl, (".py", ".libspec")
+        )
 
         self._file_changes_notifier = watchdog_wrapper.create_notifier(
             self._on_file_changed, timeout=0.5
@@ -530,7 +541,7 @@ class LibspecManager(object):
         if spec_file.lower().endswith(".libspec"):
             folder_info_on_change_spec(spec_file)
 
-    def add_workspace_folder(self, folder_uri):
+    def add_workspace_folder(self, folder_uri: str):
         self._check_in_main_thread()
         from robocorp_ls_core import uris
 
@@ -546,7 +557,7 @@ class LibspecManager(object):
         else:
             log.debug("Workspace folder already added: %s", folder_uri)
 
-    def remove_workspace_folder(self, folder_uri):
+    def remove_workspace_folder(self, folder_uri: str):
         self._check_in_main_thread()
         if folder_uri in self._workspace_folder_uri_to_folder_info:
             log.debug("Removed workspace folder: %s", folder_uri)
@@ -824,12 +835,12 @@ class LibspecManager(object):
                 else:
                     libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
 
-                log.debug(f"Obtaining mutex to generate libpsec: {libspec_filename}.")
+                log.debug(f"Obtaining mutex to generate libspec: {libspec_filename}.")
                 with timed_acquire_mutex(
                     _get_libspec_mutex_name(libspec_filename)
                 ):  # Could fail.
                     log.debug(
-                        f"Obtained mutex to generate libpsec: {libspec_filename}."
+                        f"Obtained mutex to generate libspec: {libspec_filename}."
                     )
                     call.append(libspec_filename)
 
@@ -848,7 +859,7 @@ class LibspecManager(object):
                     try:
                         try:
                             # Note: stdout is always subprocess.PIPE in this call.
-                            # Note: the env is alway inherited (the process which has
+                            # Note: the env is always inherited (the process which has
                             # the LibspecManager must be the target env already).
                             self._subprocess_check_output(
                                 call,
