@@ -242,7 +242,9 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             "workspace": {
                 "workspaceFolders": {"supported": True, "changeNotifications": True}
             },
-            "workspaceSymbolProvider": {"workDoneProgress": False},
+            "workspaceSymbolProvider": True,
+            # The one below isn't accepted by lsp4j (it's still in LSP 3.15.0).
+            # "workspaceSymbolProvider": {"workDoneProgress": False},
         }
         log.info("Server capabilities: %s", server_capabilities)
         return server_capabilities
@@ -294,6 +296,18 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             self._pm.load_plugins_from(Path(directory))
             return True
 
+        elif command == "robot.getInternalInfo":
+            in_memory_docs = []
+            workspace = self.workspace
+            if workspace:
+                for doc in workspace.iter_documents():
+                    in_memory_docs.append({"uri": doc.uri})
+            return {
+                "settings": self.config.get_full_settings(),
+                "inMemoryDocs": in_memory_docs,
+                "processId": os.getpid(),
+            }
+
         elif command == "robot.resolveInterpreter":
             try:
                 from robocorp_ls_core import uris
